==> localstack-core-2.1.1.dev20230615143138.tar.gz: localstack-core-2.1.1.dev20230615143138/localstack/services/cloudformation/models/sqs.py <==
import json
import logging

from botocore.exceptions import ClientError

from localstack.services.cloudformation.deployment_utils import (
    generate_default_name,
    params_list_to_dict,
    params_select_attributes,
)
from localstack.services.cloudformation.service_models import GenericBaseModel
from localstack.utils.aws import arns, aws_stack
from localstack.utils.common import short_uid

LOG = logging.getLogger(__name__)


class QueuePolicy(GenericBaseModel):
    @classmethod
    def cloudformation_type(cls):
        return "AWS::SQS::QueuePolicy"

    def fetch_state(self, stack_name, resources):
        if self.resource_json.get("PhysicalResourceId"):
            return {"something": "something"}  # gotta return <something> since {} is falsey :)

    @classmethod
    def get_deploy_templates(cls):
        def _create(resource_id, resources, resource_type, func, stack_name):
            sqs_client = aws_stack.connect_to_service("sqs")
            resource = cls(resources[resource_id])
            props = resource.props

            resources[resource_id]["PhysicalResourceId"] = "%s-%s-%s" % (
                stack_name,
                resource_id,
                short_uid(),
            )

            policy = json.dumps(props["PolicyDocument"])
            for queue in props["Queues"]:
                sqs_client.set_queue_attributes(QueueUrl=queue, Attributes={"Policy": policy})

        def _delete(resource_id, resources, *args, **kwargs):
            sqs_client = aws_stack.connect_to_service("sqs")
            resource = cls(resources[resource_id])
            props = resource.props

            for queue in props["Queues"]:
                try:
                    sqs_client.set_queue_attributes(QueueUrl=queue, Attributes={"Policy": ""})
                except ClientError as err:
                    if "AWS.SimpleQueueService.NonExistentQueue" != err.response["Error"]["Code"]:
                        raise

        return {
            "create": {"function": _create},
            "delete": {
                "function": _delete,
            },
        }


class SQSQueue(GenericBaseModel):
    @classmethod
    def cloudformation_type(cls):
        return "AWS::SQS::Queue"

    def get_cfn_attribute(self, attribute_name):
        if attribute_name == "Arn":
            return arns.sqs_queue_arn(self.properties["QueueName"])
        return super().get_cfn_attribute(attribute_name)

    def fetch_state(self, stack_name, resources):
        queue_name = self.props["QueueName"]
        sqs_client = aws_stack.connect_to_service("sqs")
        queues = sqs_client.list_queues()
        result = list(
            filter(
                lambda item:
                # TODO possibly find a better way to compare resource_id with queue URLs
                item.endswith("/%s" % queue_name),
                queues.get("QueueUrls", []),
            )
        )
        if not result:
            return None
        result = sqs_client.get_queue_attributes(QueueUrl=result[0], AttributeNames=["All"])[
            "Attributes"
        ]
        result["Arn"] = result["QueueArn"]
        return result

    @staticmethod
    def add_defaults(resource, stack_name: str):
        role_name = resource.get("Properties", {}).get("QueueName")

        if not role_name:
            resource["Properties"]["QueueName"] = generate_default_name(
                stack_name, resource["LogicalResourceId"]
            )
            if resource["Properties"].get("FifoQueue"):
                resource["Properties"]["QueueName"] += ".fifo"

    @classmethod
    def get_deploy_templates(cls):
        def _queue_url(params, resources, resource_id, **kwargs):
            resource = cls(resources[resource_id])
            props = resource.props
            queue_url = resource.physical_resource_id or props.get("QueueUrl")
            if queue_url:
                return queue_url
            return arns.sqs_queue_url_for_arn(props["QueueArn"])

        def _set_physical_resource_id(
            result: dict, resource_id: str, resources: dict, resource_type: str
        ):
            resources[resource_id]["PhysicalResourceId"] = result["QueueUrl"]

        return {
            "create": {
                "function": "create_queue",
                "parameters": {
                    "QueueName": "QueueName",
                    "Attributes": params_select_attributes(
                        "ContentBasedDeduplication",
                        "DelaySeconds",
                        "FifoQueue",
                        "MaximumMessageSize",
                        "MessageRetentionPeriod",
                        "VisibilityTimeout",
                        "RedrivePolicy",
                        "ReceiveMessageWaitTimeSeconds",
                    ),
                    "tags": params_list_to_dict("Tags"),
                },
                "result_handler": _set_physical_resource_id,
            },
            "delete": {
                "function": "delete_queue",
                "parameters": {"QueueUrl": _queue_url},
            },
        }

==> localstack-core-2.2.1.dev20230912121329.tar.gz: localstack-core-2.2.1.dev20230912121329/localstack/services/cloudformation/models/sqs.py <==
import json
import logging

from botocore.exceptions import ClientError

from localstack.aws.connect import connect_to
from localstack.services.cloudformation.deployment_utils import (
    generate_default_name,
    params_list_to_dict,
    params_select_attributes,
)
from localstack.services.cloudformation.service_models import GenericBaseModel
from localstack.utils.aws import arns
from localstack.utils.common import short_uid

LOG = logging.getLogger(__name__)


class QueuePolicy(GenericBaseModel):
    @classmethod
    def cloudformation_type(cls):
        return "AWS::SQS::QueuePolicy"

    def fetch_state(self, stack_name, resources):
        if self.resource_json.get("PhysicalResourceId"):
            return {"something": "something"}  # gotta return <something> since {} is falsey :)

    @classmethod
    def get_deploy_templates(cls):
        def _create(
            account_id: str,
            region_name: str,
            logical_resource_id: str,
            resource: dict,
            stack_name: str,
        ):
            sqs_client = connect_to(aws_access_key_id=account_id, region_name=region_name).sqs
            resource_provider = cls(account_id, region_name, resource)
            props = resource_provider.props

            resource["PhysicalResourceId"] = "%s-%s-%s" % (
                stack_name,
                logical_resource_id,
                short_uid(),
            )

            policy = json.dumps(props["PolicyDocument"])
            for queue in props["Queues"]:
                sqs_client.set_queue_attributes(QueueUrl=queue, Attributes={"Policy": policy})

        def _delete(
            account_id: str,
            region_name: str,
            logical_resource_id: str,
            resource: dict,
            stack_name: str,
        ):
            sqs_client = connect_to(aws_access_key_id=account_id, region_name=region_name).sqs
            resource_provider = cls(account_id, region_name, resource)
            props = resource_provider.props

            for queue in props["Queues"]:
                try:
                    sqs_client.set_queue_attributes(QueueUrl=queue, Attributes={"Policy": ""})
                except ClientError as err:
                    if "AWS.SimpleQueueService.NonExistentQueue" != err.response["Error"]["Code"]:
                        raise

        return {
            "create": {"function": _create},
            "delete": {
                "function": _delete,
            },
        }


class SQSQueue(GenericBaseModel):
    @classmethod
    def cloudformation_type(cls):
        return "AWS::SQS::Queue"

    def fetch_state(self, stack_name, resources):
        queue_name = self.props["QueueName"]
        sqs_client = connect_to(aws_access_key_id=self.account_id, region_name=self.region_name).sqs
        queues = sqs_client.list_queues()
        result = list(
            filter(
                lambda item:
                # TODO possibly find a better way to compare resource_id with queue URLs
                item.endswith("/%s" % queue_name),
                queues.get("QueueUrls", []),
            )
        )
        if not result:
            return None
        result = sqs_client.get_queue_attributes(QueueUrl=result[0], AttributeNames=["All"])[
            "Attributes"
        ]
        result["Arn"] = result["QueueArn"]
        return result

    @staticmethod
    def add_defaults(resource, stack_name: str):
        role_name = resource.get("Properties", {}).get("QueueName")

        if not role_name:
            resource["Properties"]["QueueName"] = generate_default_name(
                stack_name, resource["LogicalResourceId"]
            )
            if resource["Properties"].get("FifoQueue"):
                resource["Properties"]["QueueName"] += ".fifo"

    @classmethod
    def get_deploy_templates(cls):
        def _queue_url(
            account_id: str,
            region_name: str,
            properties: dict,
            logical_resource_id: str,
            resource: dict,
            stack_name: str,
        ):
            provider = cls(account_id, region_name, resource)
            queue_url = provider.physical_resource_id or properties.get("QueueUrl")
            if queue_url:
                return queue_url
            return arns.sqs_queue_url_for_arn(properties["QueueArn"])

        def _handle_result(
            account_id: str,
            region_name: str,
            result: dict,
            logical_resource_id: str,
            resource: dict,
        ):
            resource["PhysicalResourceId"] = result["QueueUrl"]

        return {
            "create": {
                "function": "create_queue",
                "parameters": {
                    "QueueName": "QueueName",
                    "Attributes": params_select_attributes(
                        "ContentBasedDeduplication",
                        "DelaySeconds",
                        "FifoQueue",
                        "MaximumMessageSize",
                        "MessageRetentionPeriod",
                        "VisibilityTimeout",
                        "RedrivePolicy",
                        "ReceiveMessageWaitTimeSeconds",
                    ),
                    "tags": params_list_to_dict("Tags"),
                },
                "result_handler": _handle_result,
            },
            "delete": {
                "function": "delete_queue",
                "parameters": {"QueueUrl": _queue_url},
            },
        }
